#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <SPI.h>
#include <MyConfig.h>
#include <MySensor.h>
//#include <MydebugLEDS.h>

char* sensorData[] = {"TEMP","HUM","LIGHT","DIMMER","PRESSURE","FORECAST","RAIN","RAINRATE","WIND","GUST",
                     "DIRECTON","UV","WEIGHT","DISTANCE","IMPEDANCE","ARMED","TRIPPED","WATT","KWH","SCENE_ON",
                     "SCENE_OFF","HEATER","HEATER_SW","LIGHT_LEVEL","VAR1","VAR2","VAR3","VAR4","VAR5","UP",
                     "DOWN","STOP","IR_SEND","IR_RECEIVE","FLOW","VOLUME","LOCK_STATUS","DUST_LEVEL","VOLTAGE","CURRENT",
                     "","","","","","","","","","",
                     "","","","","","","","","","",
                     "Started!\n","SKETCH_NAME","SKETCH_VERSION","UNKNOWN"};


typedef struct 
{
  int id;
  int child;
  int presentation;
  int subType;
  String protocol;
  String Payload;
} structIDs;
const int structs = 10;
structIDs IDs[structs];

LiquidCrystal_I2C lcd(0x38, 7,  POSITIVE);
/*
dLED debugLED1(4);
dLED debugLED2(5);
dLED debugLED3(6);
dLED debugLED4(7);
*/
MySensor gw;
bool notDone = true;
int nodeVisable;
int nodeOnDisplay = -1;
void setup(void)
{
  for (int i=0;i < structs;i++) {
    IDs[i].id = 255;
  }
  Serial.begin(115200);
  lcd.begin(20,4);
  gw.begin(incomingMessage, 0, true);
  gw.present(255,255);
  gw.openReadingPipe(1,BASE_RADIO_ID);
  gw.startListening();
  lcd.home ();
  lcd.setCursor ( 0, 0 );
  lcd.print("      MySensors");
  lcd.setCursor ( 0, 1 );
  lcd.print("  Node Scanner 1.1");
  lcd.setCursor ( 0, 2 );
  lcd.print("  Wait for a node.");
}

void loop(void)
{
  if (notDone) doOnce();
  else 
  gw.process();
}

void incomingMessage (const MyMessage &message) {
  lcd.backlight();
  notDone = false;
  
  Serial.print(message.sender);        Serial.print(";");
  Serial.print(message.sensor);        Serial.print(";");
  Serial.print(mGetCommand(message));  Serial.print(";");
  Serial.print(mGetAck(message));      Serial.print(";");
  Serial.print(message.type);          Serial.print(";");
  Serial.println(message.getString());

  if ((mGetCommand(message) == 0) && (message.sensor != 255)) {
    int strNr = searchID(message.sender);
    if (strNr==255) strNr = newID();
    IDs[strNr].id = message.sender;
    
    if (mGetCommand(message) == 0) {
      if (message.type != 18) IDs[strNr].child = message.sensor;
      IDs[strNr].protocol = message.getString();
      IDs[strNr].presentation = message.type;
      IDs[strNr].subType = message.type;
    }
    if (mGetCommand(message) == 1) {
      IDs[strNr].Payload = message.getString();
    }
    showNodes(strNr);
  }
}

void showNodes(int nodeNr) {
//    debugLED3.ON();
    if ((nodeOnDisplay != nodeNr) && (IDs[nodeOnDisplay].child != 255)) 
    {
//      debugLED4.ON();
      nodeOnDisplay = nodeNr;
      lcd.clear ();
      lcd.print(nodeOnDisplay);
      lcd.print(" Node ");
      lcd.print(IDs[nodeOnDisplay].id);
      lcd.print(".");
      lcd.print(IDs[nodeOnDisplay].child);
      lcd.print(" (");
      lcd.print(IDs[nodeOnDisplay].protocol);
      lcd.print(")");
      lcd.setCursor ( 2, 1 );
      lcd.print(sensorData[IDs[nodeOnDisplay].subType]);
      lcd.setCursor ( 2, 3 );
      lcd.print("Payload : ");
      lcd.print(IDs[nodeOnDisplay].Payload);
    }
}

void doOnce() {
  int tmp = 0;
  while (notDone) {
    for (int i=0;i<4;i++) {
      lcd.setCursor ( i+4, 3 );
      lcd.print("    *    ");
      delay (200);
    }
    for (int i=4;i>0;i--) {
      lcd.setCursor ( i+4, 3 );
      lcd.print("    *    ");
      delay (200);
    }
    gw.process();
    tmp++;
    if (tmp == 5) lcd.noBacklight();
  }
}

int searchID(int tmpSender) {
//  debugLED1.ON();
  for (int i=0;i<structs;i++) {
    if (IDs[i].id == tmpSender) {
      Serial.print("ID search = ");
      Serial.println(i);
      return i;
    }
    if (i == structs) return 255;
  }
}

int newID() {
// debugLED2.ON();
  for (int i=0;i<structs;i++) {
    if (IDs[i].id == 255) {
      Serial.print("ID new = ");
      Serial.println(i);
      return i;
    }
  }
}

